<?php

namespace Drupal\Tests\mixitup_views\Kernel;

use Drupal\KernelTests\KernelTestBase;

/**
 * Class MixItUpFuncTest.
 *
 * @group MixItUp Views
 * @package Drupal\Tests\mixitup_views\Kernel
 */
class MixItUpFuncTest extends KernelTestBase {

  use TestContentGenerationTrait;

  /**
   * Class to be tested.
   *
   * @var \Drupal\mixitup_views\MixitupFunc
   */
  protected $testObject;

  /**
   * EntityTypeManager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * Default options service.
   *
   * @var \Drupal\mixitup_views\MixitupViewsDefaultOptionsService
   */
  protected $defaultOptions;

  /**
   * The service container.
   *
   * @var \Symfony\Component\DependencyInjection\ContainerInterface
   */
  protected $serviceContainer;

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'system',
    'field',
    'text',
    'entity_reference',
    'user',
    'node',
    'taxonomy',
    'mixitup_views',
  ];

  /**
   * Before a test method is run, setUp() is invoked.
   *
   * Create new unit object.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function setUp(): void {
    parent::setUp();

    // Installing entities schema.
    $this->installSchema('system', ['sequences', 'key_value_expire']);
    $this->installEntitySchema('user');
    $this->installEntitySchema('taxonomy_term');
    $this->installEntitySchema('node');

    // Generating test content and creating instance for the class to be tested.
    $this->serviceContainer = \Drupal::getContainer();
    $this->entityTypeManager = $this->serviceContainer->get('entity_type.manager');
    $this->defaultOptions = $this->serviceContainer->get('mixitup_views.default_options_service');
    $this->testObject = $this->serviceContainer->get('mixitup_views.func_service');
    $this->generateTestContent();
  }

  /**
   * @covers \Drupal\mixitup_views\MixitupFunc::getRowClasses
   */
  public function testGetRowClasses() {
    // Check for equaling.
    self::assertEquals('tid_2 tid_3', $this->testObject->getRowClasses(5));
    self::assertEquals('tid_1 tid_4', $this->testObject->getRowClasses(4));
    self::assertEquals('tid_1 tid_5', $this->testObject->getRowClasses(3));
    self::assertEquals('tid_3 tid_4', $this->testObject->getRowClasses(2));
    self::assertEquals('tid_2 tid_4', $this->testObject->getRowClasses(1));
  }

  /**
   * @covers \Drupal\mixitup_views\MixitupFunc::getNodeTids
   */
  public function testGetNodeTids() {
    // Check for equaling.
    self::assertEquals([0 => 2, 1 => 3], $this->testObject->getNodeTids(5));
    self::assertEquals([0 => 1, 1 => 4], $this->testObject->getNodeTids(4));
    self::assertEquals([0 => 1, 1 => 5], $this->testObject->getNodeTids(3));
    self::assertEquals([0 => 3, 1 => 4], $this->testObject->getNodeTids(2));
    self::assertEquals([0 => 2, 1 => 4], $this->testObject->getNodeTids(1));
  }

  /**
   * @covers \Drupal\mixitup_views\MixitupFunc::populateFilters
   * @covers \Drupal\mixitup_views\MixitupFunc::getPopulatedFilters
   */
  public function testPopulateFilters() {
    $this->testObject->populateFilters(2, 5);
    $this->assertEquals(['tags' => ['.tid_2' => 'test name1']], $this->testObject->getPopulatedFilters());
  }

  /**
   * @covers \Drupal\mixitup_views\MixitupFunc::populateNodeFilters
   * @covers \Drupal\mixitup_views\MixitupFunc::getPopulatedNodeFilters
   */
  public function testPopulateNodeFilters() {
    $this->testObject->populateNodeFilters(2, 5);
    $this->assertEquals([5 => [2]], $this->testObject->getPopulatedNodeFilters());
  }

  /**
   * If test has finished running, tearDown() will be invoked.
   *
   * Unset the $testObject object.
   */
  public function tearDown(): void {
    unset($this->testObject);
  }

}
